# **********************************************************************
# Copyright 2016 VMware, Inc.  All rights reserved. VMware Confidential.
# **********************************************************************
# $Id$
# $DateTime$
# $Change$
# $Author$
# **********************************************************************
import nsx_logging
import json
import uuid

import cli.commands.host_shared.ovs_util as ovs_util

from cli.commands.cli_command_provider import CliCommandProvider

lg = nsx_logging.getLogger("cli.commands.host_kvm.mirror_session")

class MirrorSessionIdArgument(object):
    @staticmethod
    def verify(id_str, req_dict):
        try:
            uuid_val = uuid.UUID(id_str)
        except ValueError:
            return False
        return True

def strip_context_str(id_str):
    if id_str == '':
        return []

    ret = []
    entry_list = id_str.rstrip(';').split(';')
    for entry in entry_list:
        if 'Context' not in entry:
            ret.append(entry)
            continue
        ret.append(entry[:entry.find('(')])
    return ret

def get_mirror_setting(fetch_all = True):
    ms_list = []
    ms_config = ovs_util.get_mirror_session()
    for i in range(0, len(ms_config), 14):
        cfg = {}
        cfg['uuid'] = ms_config[i][0].split(':', 1)[1]
        cfg['direction'] = ms_config[i + 1][0].split(':', 1)[1]
        cfg['snap_len'] = int(ms_config[i + 2][0].split(':', 1)[1])
        if fetch_all:
            cfg['src'] = strip_context_str(ms_config[i + 3][0].split(':', 1)[1])
            cfg['dst'] = strip_context_str(ms_config[i + 4][0].split(':', 1)[1])
            cfg['EncapType'] = int(ms_config[i + 5][0].split(':', 1)[1])
            if cfg['EncapType'] == 0:
               cfg['EncapType'] = ''
               cfg['greKey'] = ''
               cfg['ERspanID'] = ''
            elif cfg['EncapType'] == 1:
               cfg['greKey'] = int(ms_config[i + 6][0].split(':', 1)[1])
               cfg['ERspanID'] = ''
               cfg['EncapType'] = 'GRE'
            elif cfg['EncapType'] == 2:
               cfg['greKey'] = ''
               cfg['ERspanID'] = int(ms_config[i + 6][0].split(':', 1)[1])
               cfg['EncapType'] = 'ERSPAN_TWO'
            else:
               cfg['greKey'] = ''
               cfg['ERspanID'] = int(ms_config[i + 6][0].split(':', 1)[1])
               cfg['EncapType'] = 'ERSPAN_THREE'
            cfg['has_filter'] = int(ms_config[i + 7][0].split(':', 1)[1])
            if cfg['has_filter']:
                cfg['src_ips'] = \
                    strip_context_str(ms_config[i + 8][0].split(':', 1)[1])
                cfg['dst_ips'] = \
                    strip_context_str(ms_config[i + 9][0].split(':', 1)[1])
                cfg['ip_protocol'] = ms_config[i + 10][0].split(':', 1)[1]
                cfg['src_ports'] = \
                    strip_context_str(ms_config[i + 11][0].split(':', 1)[1])
                cfg['dst_ports'] = \
                    strip_context_str(ms_config[i + 12][0].split(':', 1)[1])
                cfg['action'] = ms_config[i + 13][0].split(':', 1)[1]
        ms_list.append(cfg)
    return ms_list

class HostGetMirrorSessionSummary(CliCommandProvider):
    def __call__(self, cmd_list):
        ms_list = get_mirror_setting(False) # get mirror summary
        return "OK\n\n%s" % json.dumps(ms_list)

class HostGetMirrorSession(CliCommandProvider):
    def __call__(self, cmd_list):
        ms_list = get_mirror_setting() # get all mirror sessions
        for ms in ms_list:
            if ms['uuid'] == str(cmd_list[2]):
                return "OK\n\n%s" % json.dumps(ms)
        return 'MIRROR_SESSION_NOT_FOUND'
