# ***************************************************************************
# Copyright 2019 VMware, Inc.  All rights reserved. VMware Confidential.
# ***************************************************************************

import json
import os
import nsx_logging
from cli.utils.command_utils import invoke_command
from cli.utils.constants import NODE_TYPE_HOST_ESX
from cli.utils.location_utils import FilePaths
from cli.utils.node_utils import get_node_type
from cli.utils.node_utils import is_windows
from cli.utils.error_code import ErrorCode as ec

lg = nsx_logging.getLogger("cli.commands.lcp_common.netopa.client")

_NETOPA_CLI = {
    NODE_TYPE_HOST_ESX: {
        'bin': "/opt/vmware/nsx-cli/bin/nsx-appctl",
        'socket': "/var/run/vmware/nsx-netopa/netopa-cli"
    }
}


def has_netopa_cli(node_type):
    return node_type in (NODE_TYPE_HOST_ESX, )


def invoke_netopa_appctl_command(cmd_list):
    node_type = get_node_type()
    if has_netopa_cli(node_type):
        cmd = [_NETOPA_CLI[node_type]['bin'],
               "-t",
               _NETOPA_CLI[node_type]['socket']]
        cmd.extend(cmd_list)
        rc, out, err = invoke_command(cmd)
        return (rc, out, err)
    else:
        lg.error("node type %s does not have netopa CLI", node_type,
                 error_code=ec.OPERATION_NOT_SUPPORTED)
        return (-1, None, "invalid command")


def get_netopa_cli_output(command, argv=None):
    if argv is None:
        argv = []
    cmd_list = [command]

    length = len(argv)
    if length > 0:
        cmd_list.extend(argv)
    rc, out, err = invoke_netopa_appctl_command(cmd_list)
    if rc != 0:
        lg.error("Failed to invoke Netopa CLI command, rc: %d, err: %s",
                 rc, err, error_code=ec.OPERATION_FAILED)
        out = None

    return out
