# Copyright (C) 2020 VMware, Inc. All rights reserved.
# -- VMware Confidential
import json
import os
import platform
import operator
import subprocess
import traceback
import sha.config
from sha.config import is_ua
if is_ua():
    import psutil

from sha.contrib.metric.abc_base import Metric
from sha.contrib.metric.utils import run_command, timeout_cache

from sha.constants import PID, USER_NAME, NICE, NAME, VMS, RSS, SHARED, CPU_PERCENT,\
    MEMORY_PERCENT, MEM_LIST, CPU_LIST, Bytes_to_MB

class TopProcessList(Metric):

    def __init__(self):
        self._logger = sha.config.get_logger(__name__)

    @timeout_cache(2)
    def usage(self, *args, **kwargs):
        def getTopProcessData():
            '''
            Get list of running process sorted by Memory Usage and CPU percent
            '''
            listOfProcObjects = []
            processData = {}
            for proc in psutil.process_iter():
                try:
                    pinfo = proc.as_dict(attrs=[PID, NAME, USER_NAME, CPU_PERCENT, NICE, MEMORY_PERCENT])
                    pinfo[VMS] = proc.memory_info().vms / Bytes_to_MB
                    pinfo[RSS] = proc.memory_info().rss / Bytes_to_MB
                    pinfo[SHARED] = proc.memory_info().shared / Bytes_to_MB
                    listOfProcObjects.append(pinfo)
                except (psutil.NoSuchProcess, psutil.AccessDenied, psutil.ZombieProcess):
                    pass
            if listOfProcObjects:
                processData[MEM_LIST] = sorted(listOfProcObjects, key=lambda procObj: procObj[MEMORY_PERCENT], reverse=True)[:5]
                processData[CPU_LIST] = sorted(listOfProcObjects, key=lambda procObj: procObj[CPU_PERCENT], reverse=True)[:5]
            self._logger.debug("process info: {}".format(processData))
            return processData

        return getTopProcessData()