# Copyright (C) 2019-2020 VMware, Inc. All rights reserved.
# -- VMware Confidential


def stable_condition(connection_list, p):
    """
    Stable cases:
    case #1. connection_list is an empty (size = 0) list (the list should not be none, as none indicating error in metric
       collection)
    case #2. all connection statuses are HEALTHY
    """
    if connection_list is None:
        return False

    if len(connection_list) == 0:
        return True

    return all(map(lambda connection: connection['stable'] is True, connection_list))


def unstable_condition(connection_list, p):
    """
    Unstable cases:
    case #1. connection_list is none (none indicates error in metric collection)
    case #2. any connection status is HEALTHY
    """
    if connection_list is None:
        return True

    return connection_list \
           and any(map(lambda connection: connection['stable'] is False, connection_list))


def unhealthy_report(connection_list, p):
    if connection_list is None:
        return Hbreport(status='UNHEALTHY', detail="Failed in getting hyperbus connections", profile=p)

    detail_info = ''
    for connection in connection_list:
        if connection['stable'] is False:
            if not detail_info:
                detail_info = "Not all hyperbus connections are in HEALTHY: ["
            else:
                detail_info = detail_info + ","
            item = "(VIF id: " + connection['id'] + ', connection status: ' + connection['status']
            if 'detail' in connection:
                item = item + ', ' + connection['detail']
            item = item + ')'
            detail_info = detail_info + item
    return Hbreport(status='UNHEALTHY', detail=detail_info+"]", profile=p)


logger.debug('data_store:{}'.format(data_store))
metric_name = 'nsx.hyperbus.hyperbus-connection-list'
profile = data_store['profile']
evt_type_id = feature_event_type_mappings.CNI_HEALTH_HYPERBUS_MANAGER_CONNECTION_DOWN_EVENT_TYPE_ID

if data_store['metrics'][metric_name] is not None:
    connection_list = data_store['metrics'][metric_name]['value']
    if unstable_condition(connection_list, profile):
        unhealthy_report(connection_list, profile)
        Alarm(status={evt_type_id: {data_store['host_id']: (True, None)}}, feature="cni_health")

    if stable_condition(connection_list, profile):
        Hbreport(status="HEALTHY", detail="All hyperbus connections are in HEALTHY", profile=profile)
        Alarm(status={evt_type_id: {data_store['host_id']: (False, None)}}, feature="cni_health")
else:
    logger.info('Metric {} is None from whiteboard, data_store:{}'.format(metric_name, data_store))
