# Copyright (C) 2021 VMware, Inc. All rights reserved.
# -- VMware Confidential

profile = data_store['profile']
tls_stats = data_store['metrics']['nsx.edge.tls-stats']

def encap_stats(stats):
    if stats is None:
        return

    entity_type = options_pb2.MetricsEntityType.Value('TIER1')

    tls_stats_list = stats.get('tls-stats-list', [])
    if tls_stats_list is None:
        logger.info("tls_stats_monitor: tls_stats_list is None.")
        return

    for lr_tls_stats in tls_stats_list:
        lr_uuid = lr_tls_stats.get('lr_uuid', None)
        lr_stats = lr_tls_stats.get('stats', {})

        msg = metrics_agent_rpc_pb2.MetricsAgentMessage(timestamp=int(tls_stats['timestamp']),
                                                        entity = entity_type,
                                                        entity_id = str(lr_uuid))
        try:
            tmp_msg = nsx_tls_metrics_stats_pb2.NsxTlsStats(obj_id = 'obj',
                      cached_certs = lr_stats['cached_certs'],
                      cached_cert_hits = lr_stats['cached_cert_hits'],
                      cached_cert_misses = lr_stats['cached_cert_misses'],
                      open_connections = lr_stats['open_connections'],
                      total_connections = lr_stats['total_connections'],
                      decrypted_connections = lr_stats['decrypted_connections'],
                      bypassed_due_to_rules = lr_stats['bypassed_due_to_rules'],
                      bypassed_due_to_failures = lr_stats['bypassed_due_to_failures'],
                      failed_connections = lr_stats['failed_connections'],
                      client_to_server_bytes = lr_stats['client_to_server_bytes'],
                      server_to_client_bytes = lr_stats['server_to_client_bytes'])
        except KeyError:
            logger.info("Keyerror, not sending TLS to SHA for LR {}".format(lr_uuid))
            continue

        sub_msg = msg.tls.add()
        sub_msg.CopyFrom(tmp_msg)

        try:
            tmp_msg = nsx_tls_metrics_stats_pb2.NsxTlsErrors(obj_id = 'obj',
                      client_side_unexpected_message_alert = lr_stats['client_side_unexpected_message_alert'],
                      client_side_bad_record_mac_alert = lr_stats['client_side_bad_record_mac_alert'],
                      client_side_record_overflow_alert = lr_stats['client_side_record_overflow_alert'],
                      client_side_decompression_failure_alert = lr_stats['client_side_decompression_failure_alert'],
                      client_side_handshake_failure_alert = lr_stats['client_side_handshake_failure_alert'],
                      client_side_bad_certificate_alert = lr_stats['client_side_bad_certificate_alert'],
                      client_side_unsupported_certificate_alert = lr_stats['client_side_unsupported_certificate_alert'],
                      client_side_certificate_revoked_alert = lr_stats['client_side_certificate_revoked_alert'],
                      client_side_certificate_expired_alert = lr_stats['client_side_certificate_expired_alert'],
                      client_side_certificate_unknown_alert = lr_stats['client_side_certificate_unknown_alert'],
                      client_side_illegal_parameter_alert = lr_stats['client_side_illegal_parameter_alert'],
                      client_side_unknown_ca_alert = lr_stats['client_side_unknown_ca_alert'],
                      client_side_access_denied_alert = lr_stats['client_side_access_denied_alert'],
                      client_side_decode_error_alert = lr_stats['client_side_decode_error_alert'],
                      client_side_decrypt_error_alert = lr_stats['client_side_decrypt_error_alert'],
                      client_side_protocol_version_alert = lr_stats['client_side_protocol_version_alert'],
                      client_side_insufficient_security_alert = lr_stats['client_side_insufficient_security_alert'],
                      client_side_internal_error_alert = lr_stats['client_side_internal_error_alert'],
                      client_side_user_canceled_alert = lr_stats['client_side_user_canceled_alert'],
                      client_side_no_renegotiation_alert = lr_stats['client_side_no_renegotiation_alert'],
                      client_side_unsupported_extension_alert = lr_stats['client_side_unsupported_extension_alert'],
                      client_side_other_error = lr_stats['client_side_other_error'],
                      client_side_mutual_auth_required = lr_stats['client_side_mutual_auth_required'],
                      server_side_mutual_auth_required = lr_stats['server_side_mutual_auth_required'],
                      server_side_unexpected_message_alert = lr_stats['server_side_unexpected_message_alert'],
                      server_side_bad_record_mac_alert = lr_stats['server_side_bad_record_mac_alert'],
                      server_side_record_overflow_alert = lr_stats['server_side_record_overflow_alert'],
                      server_side_decompression_failure_alert = lr_stats['server_side_decompression_failure_alert'],
                      server_side_handshake_failure_alert = lr_stats['server_side_handshake_failure_alert'],
                      server_side_bad_certificate_alert = lr_stats['server_side_bad_certificate_alert'],
                      server_side_unsupported_certificate_alert = lr_stats['server_side_unsupported_certificate_alert'],
                      server_side_certificate_revoked_alert = lr_stats['server_side_certificate_revoked_alert'],
                      server_side_certificate_expired_alert = lr_stats['server_side_certificate_expired_alert'],
                      server_side_certificate_unknown_alert = lr_stats['server_side_certificate_unknown_alert'],
                      server_side_illegal_parameter_alert = lr_stats['server_side_illegal_parameter_alert'],
                      server_side_unknown_ca_alert = lr_stats['server_side_unknown_ca_alert'],
                      server_side_access_denied_alert = lr_stats['server_side_access_denied_alert'],
                      server_side_decode_error_alert = lr_stats['server_side_decode_error_alert'],
                      server_side_decrypt_error_alert = lr_stats['server_side_decrypt_error_alert'],
                      server_side_protocol_version_alert = lr_stats['server_side_protocol_version_alert'],
                      server_side_insufficient_security_alert = lr_stats['server_side_insufficient_security_alert'],
                      server_side_internal_error_alert = lr_stats['server_side_internal_error_alert'],
                      server_side_user_canceled_alert = lr_stats['server_side_user_canceled_alert'],
                      server_side_no_renegotiation_alert = lr_stats['server_side_no_renegotiation_alert'],
                      server_side_unsupported_extension_alert = lr_stats['server_side_unsupported_extension_alert'],
                      server_side_other_error = lr_stats['server_side_other_error'])
        except KeyError:
            logger.info("Keyerror, not sending TLS ERROR to SHA for LR {}".format(lr_uuid))
            continue

        sub_msg = msg.tls_errors.add()
        sub_msg.CopyFrom(tmp_msg)
        Exportdata(data = msg, exports = ['tsdb_metrics'])

    tls_rule_stats = stats.get('rule-stats', [])
    if tls_rule_stats is None:
        logger.info("tls_stats_monitor: tls_rule_stats is None.")
        return
    for rule_stats in tls_rule_stats:
        r_stats = rule_stats.get('tlsrulestat', {}).get('tls-stat', None)
        lr_uuid = rule_stats.get('mp-router-id', None)
        if_uuid = rule_stats.get('ifuuid', None)
        if_type = rule_stats.get('iftype', None)
        if r_stats == '' or r_stats is None or lr_uuid is None or if_uuid is None or \
           if_type != 'UPLINK':
            continue

        msg = metrics_agent_rpc_pb2.MetricsAgentMessage(timestamp=int(tls_stats['timestamp']),
                                                        entity = entity_type,
                                                        entity_id = str(lr_uuid))
        rule_hits = r_stats.split(';')
        for rule_hit in rule_hits:
            if re.search(r'rule', rule_hit) is None or re.search(r'hits', rule_hit) is None:
                continue
            x = rule_hit.split(':')
            y = x[0].split()
            z = x[1].split()
            if y[0] == 'rule' and z[-1] == 'hits':
                rule_id = y[1]
                hits = int(z[-2])
                tmp_msg = nsx_tls_metrics_stats_pb2.NsxTlsAggRule(obj_id = str(rule_id),
                                                                  hit_count = hits)
                sub_msg = msg.tls_agg_rule.add()
                sub_msg.CopyFrom(tmp_msg)
        Exportdata(data = msg, exports = ['tsdb_metrics'])
def handle_stats (stats):
    encap_stats(stats)
if tls_stats and tls_stats['value']:
    logger.debug("TLS_STATS stats is {}".format(tls_stats['value']))
    handle_stats(tls_stats['value'])
else:
    logger.info("NO TLS_STATS stats for exporting.")
