# Copyright (C) 2016 VMware, Inc. All rights reserved.

import struct
import logging
import sys

# TODO finalize if we need logger per module
LOGGER = logging.getLogger("nsxrpc")

# Don't complain about methods not separated by empty line in this file
# flake8: noqa: E301
class BinStream(object):
    class ByteOrder:
        NATIVE = '@'
        LE = '<'
        BE = '>'
        NETWORK = '!'  # BE

    def __init__(self, data=None, byte_order=ByteOrder.LE):
        if data is None:
            if sys.version_info[0] < 3:
                self.data = ""
            else:
                self.data = bytes()
        else:
            self.data = data
        self.byte_order = byte_order

    def __len__(self): return len(self.data)

    def read_i8(self): return self._read_('b', 1)
    def read_u8(self): return self._read_('B', 1)
    def read_i16(self): return self._read_('h', 2)
    def read_u16(self): return self._read_('H', 2)
    def read_i32(self): return self._read_('i', 4)
    def read_u32(self): return self._read_('I', 4)
    def read_data(self, len): return self._read_('%is' % len, len, '@')

    def write_i8(self, value): return self._write_('b', value)
    def write_u8(self, value): return self._write_('B', value)
    def write_i16(self, value): return self._write_('h', value)
    def write_u16(self, value): return self._write_('H', value)
    def write_i32(self, value): return self._write_('i', value)
    def write_u32(self, value): return self._write_('I', value)
    def write_data(self, value): return self._write_('%is' % len(value), value, '@')

    def _read_(self, fmt, sz, byte_order=None):
        byte_order = byte_order or self.byte_order
        fmt = byte_order + fmt
        try:
            value = struct.unpack(fmt, self.data[0:sz])[0]
            self.data = self.data[sz:]
            return value
        except:
            LOGGER.error('READ fmt: "{}", sz: {}, stream.len: {}'.format(fmt, sz, len(self.data)))
            raise

    def _write_(self, fmt, value, byte_order=None):
        byte_order = byte_order or self.byte_order
        fmt = byte_order + fmt
        try:
            self.data += struct.pack(fmt, value)
        except:
            LOGGER.error('WRITE fmt: "{}", value: "{}"'.format(fmt, value))
            raise
        return self
